package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateInput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.LineDailyElectricityTrendEnt;
import pps.core.space.entity.LineDailyElectricityTrendView;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.entity.WellDailyElectricityTrendView;
import pps.core.space.mapper.LineDailyElectricityTrendMapper;
import pps.core.space.mapper.LineDailyElectricityTrendViewMapper;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 日用电趋势计算Cloud模块
 *
 * @author ZWT
 * @date 2024/05/22 17:17
 */
@XService
public class DailyElectricityTrendCloudServiceImpl implements IDailyElectricityTrendCloudService {

    /**
     * 日用电趋势计算
     * <p>
     * (1)取昨日极短期间开优化结果
     * (2)绿电阶段开井时间：每15分钟判段光伏出力>运行功率（考虑开井顺序,功率累加） 并且当前时段开井，则算坐开井时段(0.25小时)，全天累加
     * (3)单井绿电用电：运行功率*绿电阶段开井时间
     * (4)市电用电:总用电-光伏用电
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult dailyElectricityTrendJob(XContext context) {
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //查每15分钟的有功功率
        Map<String, List<GetThirdActivePowerOutput>> powerMap = this.getAvgPhotovoltaicPower(context, yesterday);
        //查昨日间开优化结果
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap = this.getOptimizationResult(context, yesterday);
        //查昨日

        //查昨日日累计发电
        Map<String, BigDecimal> dailyUpdatePowerMap = this.getDailyUpdatePowerMap(context, yesterday);
        //查去年同期用电趋势
        Map<String, List<LineDailyElectricityTrendEnt>> lastElectricityTrendMap = this.getLastElectricityTrendMap(context, yesterday);
        //用电趋势计算结果
        List<LineDailyElectricityTrendView> saveList = new ArrayList<>(powerMap.size());
        //井口用电趋势计算结果
        List<WellDailyElectricityTrendView> wellSaveList = new ArrayList<>(powerMap.size());
        BigDecimal currentServiceRating;
        List<GetThirdActivePowerOutput> thirdActivePowerList;
        Map<Integer, List<SpaceOptimizeShortDurationView>> startSeqMap;
        List<SpaceOptimizeShortDurationView> durationList;
        GetThirdActivePowerOutput thirdActivePower;
        //遍历全量电站
        for (String stationName : powerMap.keySet()) {
            //取存在的间开
            if (lineMap.containsKey(stationName)) {
                //每15分钟有功功率
                thirdActivePowerList = powerMap.get(stationName);
                //按照启动顺序分组
                startSeqMap = lineMap.get(stationName).stream()
                        .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getStartSeq));
                //光伏日累计用电(绿电)
                BigDecimal photovoltaicPower = BigDecimal.ZERO;
                //按启动顺序取时间段列表
                BigDecimal serviceRating = BigDecimal.ZERO;
                for (int i = 1; i <= startSeqMap.size(); i++) {
                    durationList = startSeqMap.get(i);
                    //当前井运行功率
                    currentServiceRating = durationList.get(0).getServiceRating();
                    //累加运行功率
                    serviceRating.add(currentServiceRating);
                    //记录每一次计算结束位置
                    int index = 0;
                    for (SpaceOptimizeShortDurationView duration : durationList) {
                        //时间转换
                        DateTime openWellTime = DateUtil.parse(duration.getOpenWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                        DateTime closeWellTime;
                        if (CharSequenceUtil.equals(duration.getCloseWellTime(), BusinessConstant.END_OF_DAY_TIME)) {
                            closeWellTime = BusinessConstant.DATE_FLAG;
                        } else {
                            closeWellTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                        }
                        //遍历功率
                        for (int i1 = index; i1 < thirdActivePowerList.size(); i1++) {
                            thirdActivePower = thirdActivePowerList.get(i1);
                            //在时间范围内,并且总运行功率满足运行
                            if (DateUtil.isIn(thirdActivePower.getCreateDate(), openWellTime, closeWellTime) && thirdActivePower.getPhotovoltaicPower().compareTo(serviceRating) >= 0) {
                                //单井绿电用电：运行功率*绿电阶段开井时间(0.25h)
                                photovoltaicPower = photovoltaicPower.add(currentServiceRating.multiply(BusinessConstant.BIG_DECIMAL_025));
                            }
                        }
                    }
                }
                //计算封装数据

            }
        }
        //开启事务批量插入
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(saveList)) {
                LineDailyElectricityTrendViewMapper mapper = context.getBean(LineDailyElectricityTrendViewMapper.class);
                int size = saveList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<LineDailyElectricityTrendView>> subList = BaseUtils.getSubList(saveList);
                    subList.forEach(mapper::batchInsert);
                } else {
                    mapper.batchInsert(saveList);
                }
            }
            return XServiceResult.OK;
        });
    }

    /*------------------------------------- private -------------------------------------*/

    /**
     * 获取同期用电趋势信息
     *
     * @param context    上下文
     * @param createDate 创建日期
     * @return {@link Map }<{@link String }, {@link List }<{@link LineDailyElectricityTrendEnt }>>
     */
    private Map<String, List<LineDailyElectricityTrendEnt>> getLastElectricityTrendMap(XContext context, Date createDate) {
        LineDailyElectricityTrendMapper mapper = context.getBean(LineDailyElectricityTrendMapper.class);
        List<LineDailyElectricityTrendEnt> list = mapper.selectList(new LambdaQueryWrapper<LineDailyElectricityTrendEnt>()
                .apply("DATE( create_date ) = {0}", DateUtil.offset(createDate, DateField.YEAR, -1))
        );
        Map<String, List<LineDailyElectricityTrendEnt>> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(Collectors.groupingBy(LineDailyElectricityTrendEnt::getStationName));
        }
        return collect;
    }

    /**
     * 查询每日井场日累计数据
     *
     * @param context    上下文
     * @param createDate 创建日期
     * @return {@link Map }<{@link String }, {@link BigDecimal }>
     */
    private Map<String, BigDecimal> getDailyUpdatePowerMap(XContext context, Date createDate) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetThirdDailyAccumulationUpdateOutput> result = service.queryDailyUpdatePower(context, GetThirdDailyAccumulationUpdateInput.builder()
                .createDate(createDate)
                .build());
        result.throwIfFail();
        Map<String, BigDecimal> powerMap;
        List<GetThirdDailyAccumulationUpdateOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(GetThirdDailyAccumulationUpdateOutput::getStationName, GetThirdDailyAccumulationUpdateOutput::getPhotovoltaicPower));
        }
        return powerMap;
    }

    /**
     * 查每15分钟的有功功率
     *
     * @param context    上下文
     * @param createDate 创建日期
     * @return {@link Map }<{@link String }, {@link List }<{@link GetThirdActivePowerOutput }>>
     */
    private Map<String, List<GetThirdActivePowerOutput>> getAvgPhotovoltaicPower(XContext context, Date createDate) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetThirdActivePowerOutput> result = service.queryAvgPhotovoltaicPower(context, GetThirdActivePowerInput.builder()
                .createDate(createDate)
                .build());
        result.throwIfFail();
        Map<String, List<GetThirdActivePowerOutput>> powerMap;
        List<GetThirdActivePowerOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName));
        }
        return powerMap;
    }

    /**
     * 获取优化结果
     *
     * @param context      上下文
     * @param optimizeDate 优化日期
     * @return {@link Map }<{@link String }, {@link List }<{@link SpaceOptimizeShortDurationView }>>
     */
    private Map<String, List<SpaceOptimizeShortDurationView>> getOptimizationResult(XContext context, Date optimizeDate) {
        SpaceOptimizeShortDurationViewMapper shortMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationList = shortMapper.selectOptimizationResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(optimizeDate)
                .build());
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap;
        if (CollUtil.isEmpty(durationList)) {
            lineMap = Collections.emptyMap();
        } else {
            lineMap = durationList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName));
        }
        return lineMap;
    }
}
